﻿// <copyright file="WallViewModel.cs" company="Ucaya">
// </copyright>
// <author>Ucaya</author>

namespace Channel9Kinect
{
    #region

    using System;
    using System.ComponentModel;
    using System.ComponentModel.Composition;
    using System.Linq.Expressions;
    using System.Windows.Input;

    using Channel9Kinect.Input;
    using Channel9Kinect.Services;

    using MediaRss;

    #endregion

    public class WallViewModel : INotifyPropertyChanged
    {
        #region Constants and Fields

        private Uri defaultImage;

        private Uri feedUri;

        #endregion

        #region Constructors and Destructors

        /// <summary>
        ///   Initializes a new instance of the <see cref = "MainViewModel" /> class.
        /// </summary>
        public WallViewModel()
        {
            ServiceLocator.Instance.SatisfyImportsOnce(this);
            this.PlayItem = new Command<MediaRssItem>(item =>
                {
                    this.NavigationServiceWrapper.NavigationService.NavigateTo(new Uri("Player.xaml", UriKind.Relative), item);
                });

            this.Back = new Command(() => this.NavigationServiceWrapper.NavigationService.GoBack());
        }

        #endregion

        #region Public Events

        /// <summary>
        ///   The property changed.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        #region Public Properties

        public ICommand Back { get; private set; }

        public Uri DefaultImage
        {
            get
            {
                return this.defaultImage;
            }
            set
            {
                if (this.defaultImage != value)
                {
                    this.defaultImage = value;
                    this.RaisePropertyChanged(n => n.DefaultImage);
                }
            }
        }

        public Uri FeedUri
        {
            get
            {
                return this.feedUri;
            }
            set
            {
                if (this.feedUri != value)
                {
                    this.feedUri = value;
                    this.RaisePropertyChanged(n => n.FeedUri);
                }
            }
        }

        [Import]
        public NavigationServiceWrapper NavigationServiceWrapper { get; set; }

        public Command<MediaRssItem> PlayItem { get; private set; }

        #endregion

        #region Methods

        /// <summary>
        ///   The raise property changed.
        /// </summary>
        /// <param name = "expression">
        ///   The expression.
        /// </param>
        /// <typeparam name = "TProperty">
        /// </typeparam>
        protected void RaisePropertyChanged<TProperty>(Expression<Func<WallViewModel, TProperty>> expression)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(this.GetPropertyName(expression)));
            }
        }

        #endregion
    }
}