﻿// <copyright file="Content.cs" company="Ucaya">
// </copyright>
// <author>Ucaya</author>

namespace MediaRss
{
    #region

    using System;
    using System.Text;
    using System.Xml;
    using System.Xml.Serialization;

    #endregion

    [XmlRoot(ElementName = "content", Namespace = "http://search.yahoo.com/mrss/")]
    public class Content : MediaRssBase, IXmlSerializable
    {
        #region Constants and Fields

        internal const string ELEMENT_NAME = "content";

        #endregion

        #region Public Properties

        public int BitRate { get; set; }

        public string Expression { get; set; }

        public double FileSize { get; set; }

        public string Type { get; set; }

        public Uri Url { get; set; }

        #endregion

        #region Public Methods

        public void ReadXml(XmlReader reader)
        {
            bool isEmptyElement = reader.IsEmptyElement;
            if (reader.HasAttributes)
            {
                for (int i = 0; i < reader.AttributeCount; i++)
                {
                    reader.MoveToNextAttribute();
                    if (reader.NamespaceURI == "")
                    {
                        if (reader.LocalName == "url")
                        {
                            this.Url = new Uri(reader.Value);
                        }
                        else if (reader.LocalName == "fileSize")
                        {
                            this.FileSize = double.Parse(reader.Value);
                        }
                        else if (reader.LocalName == "bitrate")
                        {
                            this.BitRate = int.Parse(reader.Value);
                        }
                        else if (reader.LocalName == "type")
                        {
                            this.Type = reader.Value;
                        }
                        else if (reader.LocalName == "expression")
                        {
                            this.Expression = reader.Value;
                        }
                        else
                        {
                            base.AttributeExtensions.Add(new XmlQualifiedName(reader.LocalName, reader.NamespaceURI), reader.Value);
                        }
                    }
                }
            }
            reader.ReadStartElement();
            base.AddElementExtensions(reader, isEmptyElement);
        }

        public override string ToString()
        {
            var builder = new StringBuilder();
            builder.AppendFormat("Url: {0}\n", this.Url);
            builder.AppendFormat("FileSize: {0}\n", this.FileSize);
            builder.AppendFormat("BitRate: {0}\n", this.BitRate);
            builder.AppendFormat("Type: {0}\n", this.Type);
            builder.AppendFormat("Expression: {0}\n", this.Expression);
            if (base.ElementExtensions.Count > 0)
            {
                foreach (var element in base.ElementExtensions)
                {
                    builder.AppendLine(element.ToString());
                }
            }
            return builder.ToString();
        }

        public void WriteXml(XmlWriter writer)
        {
            if (this.Url != null)
            {
                writer.WriteAttributeString("url", null, this.Url.ToString());
            }
            writer.WriteAttributeString("filesize", null, this.FileSize.ToString());
            writer.WriteAttributeString("bitrate", null, this.BitRate.ToString());
            if (this.Type != null)
            {
                writer.WriteAttributeString("type", null, this.Type);
            }
            if (this.Expression != null)
            {
                writer.WriteAttributeString("expression", null, this.Expression);
            }
            foreach (var pair in base.AttributeExtensions)
            {
                writer.WriteAttributeString(pair.Key.Name, pair.Key.Namespace, pair.Value);
            }
            foreach (var element in base.ElementExtensions)
            {
                element.WriteTo(writer);
            }
        }

        #endregion
    }
}