﻿// <copyright file="License.cs" company="Ucaya">
// </copyright>
// <author>Ucaya</author>

namespace MediaRss
{
    #region

    using System;
    using System.Text;
    using System.Xml;
    using System.Xml.Serialization;

    #endregion

    [XmlRoot(ElementName = "license", Namespace = "http://search.yahoo.com/mrss/")]
    public class License : MediaRssBase, IXmlSerializable
    {
        #region Constants and Fields

        internal const string ELEMENT_NAME = "license";

        #endregion

        #region Public Properties

        public string Content { get; set; }

        public Uri Href { get; set; }

        public string Type { get; set; }

        #endregion

        #region Public Methods

        public void ReadXml(XmlReader reader)
        {
            bool isEmptyElement = reader.IsEmptyElement;
            if (reader.HasAttributes)
            {
                for (int i = 0; i < reader.AttributeCount; i++)
                {
                    reader.MoveToNextAttribute();
                    if (reader.NamespaceURI == "")
                    {
                        if (reader.LocalName == "type")
                        {
                            this.Type = reader.Value;
                        }
                        else if (reader.LocalName == "href")
                        {
                            this.Href = new Uri(reader.Value);
                        }
                        else
                        {
                            base.AttributeExtensions.Add(new XmlQualifiedName(reader.LocalName, reader.NamespaceURI), reader.Value);
                        }
                    }
                }
            }
            reader.ReadStartElement();
            if (!isEmptyElement)
            {
                this.Content = reader.ReadContentAsString().Trim();
            }
            reader.ReadEndElement();
        }

        public override string ToString()
        {
            var builder = new StringBuilder();
            builder.AppendFormat("Type: {0}\n", this.Type);
            builder.AppendFormat("Href: {0}\n", this.Href);
            return builder.ToString();
        }

        public void WriteXml(XmlWriter writer)
        {
            if (this.Type != null)
            {
                writer.WriteAttributeString("type", null, this.Type);
            }
            if (this.Href != null)
            {
                writer.WriteAttributeString("href", null, this.Href.ToString());
            }
            if (!string.IsNullOrEmpty(this.Content))
            {
                writer.WriteString(this.Content);
            }
            foreach (var pair in base.AttributeExtensions)
            {
                writer.WriteAttributeString(pair.Key.Name, pair.Key.Namespace, pair.Value);
            }
        }

        #endregion
    }
}