﻿// <copyright file="Price.cs" company="Ucaya">
// </copyright>
// <author>Ucaya</author>

namespace MediaRss
{
    #region

    using System.Text;
    using System.Xml;
    using System.Xml.Serialization;

    #endregion

    [XmlRoot(ElementName = "price", Namespace = "http://search.yahoo.com/mrss/")]
    public class Price : MediaRssBase, IXmlSerializable
    {
        #region Constants and Fields

        internal const string ELEMENT_NAME = "price";

        #endregion

        #region Public Properties

        public string Currency { get; set; }

        public double PriceValue { get; set; }

        public string Type { get; set; }

        #endregion

        #region Public Methods

        public void ReadXml(XmlReader reader)
        {
            bool isEmptyElement = reader.IsEmptyElement;
            if (reader.HasAttributes)
            {
                for (int i = 0; i < reader.AttributeCount; i++)
                {
                    reader.MoveToNextAttribute();
                    if (reader.NamespaceURI == "")
                    {
                        if (reader.LocalName == "type")
                        {
                            this.Type = reader.Value;
                        }
                        else if (reader.LocalName == "price")
                        {
                            this.PriceValue = double.Parse(reader.Value);
                        }
                        else if (reader.LocalName == "currency")
                        {
                            this.Currency = reader.Value;
                        }
                        else
                        {
                            base.AttributeExtensions.Add(new XmlQualifiedName(reader.LocalName, reader.NamespaceURI), reader.Value);
                        }
                    }
                }
            }
            reader.ReadStartElement();
            base.AddElementExtensions(reader, isEmptyElement);
        }

        public override string ToString()
        {
            var builder = new StringBuilder();
            builder.AppendFormat("Type: {0}\n", this.Type);
            builder.AppendFormat("Price: {0}\n", this.PriceValue);
            builder.AppendFormat("Currency: {0}\n", this.Currency);
            return builder.ToString();
        }

        public void WriteXml(XmlWriter writer)
        {
            if (!string.IsNullOrEmpty(this.Type))
            {
                writer.WriteAttributeString("type", null, this.Type);
            }
            writer.WriteAttributeString("price", null, this.Type);
            if (this.Currency != null)
            {
                writer.WriteAttributeString("currency", null, this.Type);
            }
            foreach (var pair in base.AttributeExtensions)
            {
                writer.WriteAttributeString(pair.Key.Name, pair.Key.Namespace, pair.Value);
            }
        }

        #endregion
    }
}