﻿// <copyright file="Thumbnail.cs" company="Ucaya">
// </copyright>
// <author>Ucaya</author>

namespace MediaRss
{
    #region

    using System;
    using System.Xml;
    using System.Xml.Serialization;

    #endregion

    [XmlRoot(ElementName = "thumbnail", Namespace = "http://search.yahoo.com/mrss/")]
    public class Thumbnail : MediaRssBase, IXmlSerializable
    {
        #region Constants and Fields

        internal const string ELEMENT_NAME = "thumbnail";

        #endregion

        #region Public Properties

        public int Height { get; set; }

        public Uri ThumbnailUri { get; set; }

        public int Width { get; set; }

        #endregion

        #region Public Methods

        public void ReadXml(XmlReader reader)
        {
            bool isEmptyElement = reader.IsEmptyElement;
            if (reader.HasAttributes)
            {
                for (int i = 0; i < reader.AttributeCount; i++)
                {
                    reader.MoveToNextAttribute();
                    if (reader.NamespaceURI == "")
                    {
                        if (reader.LocalName == "url")
                        {
                            this.ThumbnailUri = new Uri(reader.Value);
                        }
                        else if (reader.LocalName == "height")
                        {
                            this.Height = int.Parse(reader.Value);
                        }
                        else if (reader.LocalName == "bitrate")
                        {
                            this.Width = int.Parse(reader.Value);
                        }

                        else
                        {
                            base.AttributeExtensions.Add(new XmlQualifiedName(reader.LocalName, reader.NamespaceURI), reader.Value);
                        }
                    }
                }
            }
            reader.ReadStartElement();
            base.AddElementExtensions(reader, isEmptyElement);
        }

        public void WriteXml(XmlWriter writer)
        {
            writer.WriteStartElement("media", "thumbnail", "http://search.yahoo.com/mrss/");
            foreach (var pair in base.AttributeExtensions)
            {
                writer.WriteAttributeString(pair.Key.Name, pair.Key.Namespace, pair.Value);
            }
            if (this.ThumbnailUri != null)
            {
                writer.WriteString(this.ThumbnailUri.ToString());
            }
            writer.WriteEndElement();
            foreach (var element in base.ElementExtensions)
            {
                element.WriteTo(writer);
            }
        }

        #endregion
    }
}